@@ -1,30 +1,45 @@
 use crate::{datetime::*, parse::*, table::*, *};
+use libc::pid_t;
 use std::{collections::{BTreeMap, HashMap, HashSet},
           io::{stdin, IsTerminal}};
 
 /// Straightforward display of merge events
 ///
 /// We store the start times in a hashmap to compute/print the duration when we reach a stop event.
-pub fn cmd_log(gc: &Conf, sc: &ConfLog) -> Result<bool, Error> {
+pub fn cmd_log(gc: Conf, sc: ConfLog) -> Result<bool, Error> {
     let hist = get_hist(&gc.logfile, gc.from, gc.to, sc.show, &sc.search, sc.exact)?;
-    let mut merges: HashMap<String, i64> = HashMap::new();
+    let mut merges: HashMap<String, (i64, bool)> = HashMap::new();
     let mut unmerges: HashMap<String, i64> = HashMap::new();
-    let mut found = 0;
     let mut sync_start: Option<i64> = None;
-    let mut tbl = Table::new(gc).align_left(0).align_left(2).margin(2, " ").last(sc.last);
-    tbl.header(["Date", "Duration", "Package/Repo"]);
+    let mut found = 0;
+    let h = ["Date", "Duration", "Package/Repo"];
+    let mut tbl =
+        Table::new(&gc).align_left(0).align_left(2).margin(2, " ").last(sc.last).header(h);
     for p in hist {
         match p {
+            Hist::RunStart { ts, args, .. } => {
+                found += 1;
+                if found <= sc.first {
+                    tbl.row([&[&FmtDate(ts)], &[], &[&"Emerge ", &args]]);
+                }
+            },
             Hist::MergeStart { ts, key, .. } => {
                 // This'll overwrite any previous entry, if a merge started but never finished
-                merges.insert(key, ts);
+                merges.insert(key, (ts, false));
+            },
+            Hist::MergeBin { key, .. } => {
+                if let Some((_, bin)) = merges.get_mut(&key) {
+                    *bin = true;
+                }
             },
             Hist::MergeStop { ts, ref key, .. } => {
                 found += 1;
-                let started = merges.remove(key).unwrap_or(ts + 1);
-                tbl.row([&[&FmtDate(if sc.starttime { started } else { ts })],
-                         &[&FmtDur(ts - started)],
-                         &[&gc.merge, &p.ebuild_version()]]);
+                let (started, bin) = merges.remove(key).unwrap_or((ts + 1, false));
+                if found <= sc.first {
+                    tbl.row([&[&FmtDate(if sc.starttime { started } else { ts })],
+                             &[&FmtDur(ts - started)],
+                             &[if bin { &gc.binmerge } else { &gc.merge }, &p.ebuild_version()]]);
+                }
             },
             Hist::UnmergeStart { ts, key, .. } => {
                 // This'll overwrite any previous entry, if an unmerge started but never finished
@@ -33,29 +48,33 @@ pub fn cmd_log(gc: &Conf, sc: &ConfLog) -> Result<bool, Error> {
             Hist::UnmergeStop { ts, ref key, .. } => {
                 found += 1;
                 let started = unmerges.remove(key).unwrap_or(ts + 1);
-                tbl.row([&[&FmtDate(if sc.starttime { started } else { ts })],
-                         &[&FmtDur(ts - started)],
-                         &[&gc.unmerge, &p.ebuild_version()]]);
+                if found <= sc.first {
+                    tbl.row([&[&FmtDate(if sc.starttime { started } else { ts })],
+                             &[&FmtDur(ts - started)],
+                             &[&gc.unmerge, &p.ebuild_version()]]);
+                }
             },
             Hist::SyncStart { ts } => {
                 // Some sync starts have multiple entries in old logs
                 sync_start = Some(ts);
             },
             Hist::SyncStop { ts, repo } => {
-                if let Some(started) = sync_start.take() {
-                    found += 1;
+                found += 1;
+                let started = sync_start.take().unwrap_or(ts + 1);
+                if found <= sc.first {
                     tbl.row([&[&FmtDate(if sc.starttime { started } else { ts })],
                              &[&FmtDur(ts - started)],
-                             &[&gc.clr, &"Sync ", &repo]]);
-                } else {
-                    warn!("Sync stop without a start at {ts}");
+                             &[&gc.sync, &"Sync ", &repo]]);
                 }
             },
         }
-        if found >= sc.first {
+        if !gc.showskip && found >= sc.first {
             break;
         }
     }
+    if gc.showskip && found >= sc.first {
+        tbl.skiprow(&[&gc.skip, &"(skip last ", &(found - sc.first), &")"]);
+    }
     Ok(found > 0)
 }
 
@@ -129,36 +148,69 @@ impl Times {
     }
 }
 
+/// Classify emerge commands by looking at their args.
+///
+/// Note that some commands don't get logged at all, so this enum is quite limited.
+#[derive(PartialEq, Eq, PartialOrd, Ord)]
+enum ArgKind {
+    All,
+    Merge,
+    Clean,
+    Sync,
+}
+impl ArgKind {
+    fn new(args: &str) -> Self {
+        for arg in args.split_ascii_whitespace() {
+            match arg {
+                "--deselect" | "--unmerge" | "--clean" | "--depclean" => return Self::Clean,
+                "--sync" => return Self::Sync,
+                _ => (),
+            }
+        }
+        Self::Merge
+    }
+}
+
 /// Summary display of merge events
 ///
 /// First loop is like cmd_list but we store the merge time for each ebuild instead of printing it.
 /// Then we compute the stats per ebuild, and print that.
-pub fn cmd_stats(gc: &Conf, sc: &ConfStats) -> Result<bool, Error> {
+pub fn cmd_stats(gc: Conf, sc: ConfStats) -> Result<bool, Error> {
     let hist = get_hist(&gc.logfile, gc.from, gc.to, sc.show, &sc.search, sc.exact)?;
-    let mut tbls = Table::new(gc).align_left(0).align_left(1).margin(1, " ");
-    tbls.header([sc.group.name(), "Repo", "Syncs", "Total time", "Predict time"]);
-    let mut tblp = Table::new(gc).align_left(0).align_left(1).margin(1, " ");
-    tblp.header([sc.group.name(),
-                 "Package",
-                 "Merges",
-                 "Total time",
-                 "Predict time",
-                 "Unmerges",
-                 "Total time",
-                 "Predict time"]);
-    let mut tblt = Table::new(gc).align_left(0).margin(1, " ");
-    tblt.header([sc.group.name(),
-                 "Merges",
-                 "Total time",
-                 "Average time",
-                 "Unmerges",
-                 "Total time",
-                 "Average time"]);
-    let mut merge_start: HashMap<String, i64> = HashMap::new();
+    let moves = PkgMoves::new(&Mtimedb::new());
+    let h = [sc.group.name(), "Logged emerges", "Install/Update", "Unmerge/Clean", "Sync"];
+    let mut tblc = Table::new(&gc).margin(1, " ").header(h);
+    let h = [sc.group.name(), "Repo", "Syncs", "Total time", "Predict time"];
+    let mut tbls = Table::new(&gc).align_left(0).align_left(1).margin(1, " ").header(h);
+    let h = [sc.group.name(),
+             "Package",
+             "Merges",
+             "Total time",
+             "Predict time",
+             "Binmerges",
+             "Total time",
+             "Predict time",
+             "Unmerges",
+             "Total time",
+             "Predict time"];
+    let mut tblp = Table::new(&gc).align_left(0).align_left(1).margin(1, " ").header(h);
+    let h = [sc.group.name(),
+             "Merges",
+             "Total time",
+             "Average time",
+             "Binmerges",
+             "Total time",
+             "Average time",
+             "Unmerges",
+             "Total time",
+             "Average time"];
+    let mut tblt = Table::new(&gc).align_left(0).margin(1, " ").header(h);
+    let mut merge_start: HashMap<String, (i64, bool)> = HashMap::new();
     let mut unmerge_start: HashMap<String, i64> = HashMap::new();
-    let mut pkg_time: BTreeMap<String, (Times, Times)> = BTreeMap::new();
+    let mut pkg_time: BTreeMap<String, (Times, Times, Times)> = BTreeMap::new();
     let mut sync_start: Option<i64> = None;
     let mut sync_time: BTreeMap<String, Times> = BTreeMap::new();
+    let mut run_args: BTreeMap<ArgKind, usize> = BTreeMap::new();
     let mut nextts = 0;
     let mut curts = 0;
     for p in hist {
@@ -169,32 +221,48 @@ pub fn cmd_stats(gc: &Conf, sc: &ConfStats) -> Result<bool, Error> {
                 curts = t;
             } else if t > nextts {
                 let group = sc.group.at(curts, gc.date_offset);
-                cmd_stats_group(gc, sc, &mut tbls, &mut tblp, &mut tblt, group, &sync_time,
-                                &pkg_time);
+                cmd_stats_group(&gc, &sc, &mut tblc, &mut tbls, &mut tblp, &mut tblt, group,
+                                &run_args, &sync_time, &pkg_time);
                 sync_time.clear();
                 pkg_time.clear();
+                run_args.clear();
                 nextts = sc.group.next(t, gc.date_offset);
                 curts = t;
             }
         }
         match p {
+            Hist::RunStart { args, .. } => {
+                *run_args.entry(ArgKind::All).or_insert(0) += 1;
+                *run_args.entry(ArgKind::new(&args)).or_insert(0) += 1;
+            },
             Hist::MergeStart { ts, key, .. } => {
-                merge_start.insert(key, ts);
+                merge_start.insert(moves.get(key), (ts, false));
+            },
+            Hist::MergeBin { key, .. } => {
+                if let Some((_, bin)) = merge_start.get_mut(&key) {
+                    *bin = true;
+                }
             },
             Hist::MergeStop { ts, ref key, .. } => {
-                if let Some(start_ts) = merge_start.remove(key) {
-                    let (times, _) = pkg_time.entry(p.ebuild().to_owned())
-                                             .or_insert((Times::new(), Times::new()));
-                    times.insert(ts - start_ts);
+                if let Some((start_ts, bin)) = merge_start.remove(moves.get_ref(key)) {
+                    let (tc, tb, _) =
+                        pkg_time.entry(moves.get(p.take_ebuild()))
+                                .or_insert((Times::new(), Times::new(), Times::new()));
+                    if bin {
+                        tb.insert(ts - start_ts);
+                    } else {
+                        tc.insert(ts - start_ts);
+                    }
                 }
             },
             Hist::UnmergeStart { ts, key, .. } => {
-                unmerge_start.insert(key, ts);
+                unmerge_start.insert(moves.get(key), ts);
             },
             Hist::UnmergeStop { ts, ref key, .. } => {
-                if let Some(start_ts) = unmerge_start.remove(key) {
-                    let (_, times) = pkg_time.entry(p.ebuild().to_owned())
-                                             .or_insert((Times::new(), Times::new()));
+                if let Some(start_ts) = unmerge_start.remove(moves.get_ref(key)) {
+                    let (_, _, times) =
+                        pkg_time.entry(moves.get(p.take_ebuild()))
+                                .or_insert((Times::new(), Times::new(), Times::new()));
                     times.insert(ts - start_ts);
                 }
             },
@@ -213,15 +281,20 @@ pub fn cmd_stats(gc: &Conf, sc: &ConfStats) -> Result<bool, Error> {
         }
     }
     let group = sc.group.at(curts, gc.date_offset);
-    cmd_stats_group(gc, sc, &mut tbls, &mut tblp, &mut tblt, group, &sync_time, &pkg_time);
+    cmd_stats_group(&gc, &sc, &mut tblc, &mut tbls, &mut tblp, &mut tblt, group, &run_args,
+                    &sync_time, &pkg_time);
     // Controlled drop to ensure table order and insert blank lines
-    let (es, ep, et) = (!tbls.is_empty(), !tblp.is_empty(), !tblt.is_empty());
+    let (ec, es, ep, et) = (!tblc.is_empty(), !tbls.is_empty(), !tblp.is_empty(), !tblt.is_empty());
+    drop(tblc);
+    if ec && es {
+        println!();
+    }
     drop(tbls);
-    if es && ep {
+    if (ec || es) && ep {
         println!();
     }
     drop(tblp);
-    if (es || ep) && et {
+    if (ec || es || ep) && et {
         println!();
     }
     drop(tblt);
@@ -232,17 +305,27 @@ pub fn cmd_stats(gc: &Conf, sc: &ConfStats) -> Result<bool, Error> {
 #[allow(clippy::too_many_arguments)]
 fn cmd_stats_group(gc: &Conf,
                    sc: &ConfStats,
+                   tblc: &mut Table<5>,
                    tbls: &mut Table<5>,
-                   tblp: &mut Table<8>,
-                   tblt: &mut Table<7>,
+                   tblp: &mut Table<11>,
+                   tblt: &mut Table<10>,
                    group: String,
+                   run_args: &BTreeMap<ArgKind, usize>,
                    sync_time: &BTreeMap<String, Times>,
-                   pkg_time: &BTreeMap<String, (Times, Times)>) {
+                   pkg_time: &BTreeMap<String, (Times, Times, Times)>) {
+    // Commands
+    if sc.show.run && !run_args.is_empty() {
+        tblc.row([&[&group],
+                  &[&gc.cnt, run_args.get(&ArgKind::All).unwrap_or(&0)],
+                  &[&gc.cnt, run_args.get(&ArgKind::Merge).unwrap_or(&0)],
+                  &[&gc.cnt, run_args.get(&ArgKind::Clean).unwrap_or(&0)],
+                  &[&gc.cnt, run_args.get(&ArgKind::Sync).unwrap_or(&0)]]);
+    }
     // Syncs
     if sc.show.sync && !sync_time.is_empty() {
         for (repo, time) in sync_time {
             tbls.row([&[&group],
-                      &[repo],
+                      &[&gc.sync, repo],
                       &[&gc.cnt, &time.count],
                       &[&FmtDur(time.tot)],
                       &[&FmtDur(time.pred(sc.lim, sc.avg))]]);
@@ -250,12 +333,15 @@ fn cmd_stats_group(gc: &Conf,
     }
     // Packages
     if sc.show.pkg && !pkg_time.is_empty() {
-        for (pkg, (merge, unmerge)) in pkg_time {
+        for (pkg, (merge, binmerge, unmerge)) in pkg_time {
             tblp.row([&[&group],
                       &[&gc.pkg, pkg],
                       &[&gc.cnt, &merge.count],
                       &[&FmtDur(merge.tot)],
                       &[&FmtDur(merge.pred(sc.lim, sc.avg))],
+                      &[&gc.cnt, &binmerge.count],
+                      &[&FmtDur(binmerge.tot)],
+                      &[&FmtDur(binmerge.pred(sc.lim, sc.avg))],
                       &[&gc.cnt, &unmerge.count],
                       &[&FmtDur(unmerge.tot)],
                       &[&FmtDur(unmerge.pred(sc.lim, sc.avg))]]);
@@ -265,11 +351,15 @@ fn cmd_stats_group(gc: &Conf,
     if sc.show.tot && !pkg_time.is_empty() {
         let mut merge_time = 0;
         let mut merge_count = 0;
+        let mut binmerge_time = 0;
+        let mut binmerge_count = 0;
         let mut unmerge_time = 0;
         let mut unmerge_count = 0;
-        for (merge, unmerge) in pkg_time.values() {
+        for (merge, binmerge, unmerge) in pkg_time.values() {
             merge_time += merge.tot;
             merge_count += merge.count;
+            binmerge_time += binmerge.tot;
+            binmerge_count += binmerge.count;
             unmerge_time += unmerge.tot;
             unmerge_count += unmerge.count;
         }
@@ -277,60 +367,115 @@ fn cmd_stats_group(gc: &Conf,
                   &[&gc.cnt, &merge_count],
                   &[&FmtDur(merge_time)],
                   &[&FmtDur(merge_time.checked_div(merge_count).unwrap_or(-1))],
+                  &[&gc.cnt, &binmerge_count],
+                  &[&FmtDur(binmerge_time)],
+                  &[&FmtDur(binmerge_time.checked_div(binmerge_count).unwrap_or(-1))],
                   &[&gc.cnt, &unmerge_count],
                   &[&FmtDur(unmerge_time)],
                   &[&FmtDur(unmerge_time.checked_div(unmerge_count).unwrap_or(-1))]]);
     }
 }
 
+/// Count processes in tree, including given proces
+fn proc_count(procs: &ProcList, pid: pid_t) -> usize {
+    let mut count = 1;
+    for child in procs.iter().filter(|(_, p)| p.ppid == pid).map(|(pid, _)| pid) {
+        count += proc_count(procs, *child);
+    }
+    count
+}
+
+/// Display proces tree
+fn proc_rows(now: i64,
+             tbl: &mut Table<3>,
+             procs: &ProcList,
+             pid: pid_t,
+             depth: usize,
+             gc: &Conf,
+             sc: &ConfPred) {
+    // This should always succeed because we're getting pid from procs, but to allow experiments we
+    // warn instead of panic/ignore.
+    let proc = match procs.get(&pid) {
+        Some(p) => p,
+        None => {
+            error!("Could not find proces {pid}");
+            return;
+        },
+    };
+    // Print current level
+    if depth < sc.pdepth {
+        tbl.row([&[&FmtProc(proc, depth, sc.pwidth)], &[&FmtDur(now - proc.start)], &[]]);
+    }
+    // Either recurse with children...
+    if depth + 1 < sc.pdepth {
+        for child in procs.iter().filter(|(_, p)| p.ppid == pid).map(|(pid, _)| pid) {
+            proc_rows(now, tbl, procs, *child, depth + 1, gc, sc);
+        }
+    }
+    // ...or print skipped rows
+    else if gc.showskip {
+        let count = proc_count(procs, pid) - 1;
+        if count > 0 {
+            tbl.skiprow(&[&"  ".repeat(depth + 1), &gc.skip, &"(skip ", &count, &" below)"]);
+        }
+    }
+}
+
 /// Predict future merge time
 ///
 /// Very similar to cmd_summary except we want total build time for a list of ebuilds.
-pub fn cmd_predict(gc: &Conf, sc: &ConfPred) -> Result<bool, Error> {
+pub fn cmd_predict(gc: Conf, mut sc: ConfPred) -> Result<bool, Error> {
     let now = epoch_now();
     let last = if sc.show.tot { sc.last.saturating_add(1) } else { sc.last };
-    let mut tbl = Table::new(gc).align_left(0).align_left(2).margin(2, " ").last(last);
-    // TODO: should be able to extend inside sc
-    let mut tmpdirs = sc.tmpdirs.clone();
+    let mut tbl = Table::new(&gc).align_left(0).align_left(2).margin(2, " ").last(last);
 
     // Gather and print info about current merge process.
-    let einfo = get_emerge(&mut tmpdirs);
-    if einfo.cmds.is_empty()
+    let procs = get_all_proc(&mut sc.tmpdirs);
+    let einfo = get_emerge(&procs);
+    if einfo.roots.is_empty()
        && std::io::stdin().is_terminal()
        && matches!(sc.resume, ResumeKind::No | ResumeKind::Auto)
     {
         tbl.row([&[&"No ongoing merge found"], &[], &[]]);
         return Ok(false);
     }
-    if sc.show.emerge {
-        for proc in &einfo.cmds {
-            tbl.row([&[&proc], &[&FmtDur(now - proc.start)], &[]]);
+    if sc.show.run {
+        for p in einfo.roots {
+            proc_rows(now, &mut tbl, &procs, p, 0, &gc, &sc);
         }
     }
 
     // Parse emerge log.
     let hist = get_hist(&gc.logfile, gc.from, gc.to, Show::m(), &vec![], false)?;
-    let mut started: BTreeMap<Pkg, i64> = BTreeMap::new();
-    let mut times: HashMap<String, Times> = HashMap::new();
+    let mdb = Mtimedb::new();
+    let moves = PkgMoves::new(&mdb);
+    let mut started: BTreeMap<String, (i64, bool)> = BTreeMap::new();
+    let mut times: HashMap<(String, bool), Times> = HashMap::new();
     for p in hist {
         match p {
-            Hist::MergeStart { ts, .. } => {
-                started.insert(Pkg::new(p.ebuild(), p.version()), ts);
+            Hist::MergeStart { ts, key, .. } => {
+                started.insert(moves.get(key), (ts, false));
             },
-            Hist::MergeStop { ts, .. } => {
-                if let Some(start_ts) = started.remove(&Pkg::new(p.ebuild(), p.version())) {
-                    let timevec = times.entry(p.ebuild().to_string()).or_insert(Times::new());
+            Hist::MergeBin { key, .. } => {
+                if let Some((_, bin)) = started.get_mut(moves.get_ref(&key)) {
+                    *bin = true;
+                }
+            },
+            Hist::MergeStop { ts, ref key, .. } => {
+                if let Some((start_ts, bin)) = started.remove(moves.get_ref(key)) {
+                    let timevec =
+                        times.entry((moves.get(p.take_ebuild()), bin)).or_insert(Times::new());
                     timevec.insert(ts - start_ts);
                 }
             },
-            _ => unreachable!("Should only receive Hist::{{Start,Stop}}"),
+            _ => unreachable!("Should only receive Hist::{{Start,Step,Stop}}"),
         }
     }
 
     // Build list of pending merges
     let pkgs: Vec<Pkg> = if std::io::stdin().is_terminal() {
         // From resume list
-        let mut r = get_resume(sc.resume);
+        let mut r = get_resume(sc.resume, &mdb);
         // Plus specific emerge processes
         for p in einfo.pkgs.iter() {
             if !r.contains(p) {
@@ -339,9 +484,9 @@ pub fn cmd_predict(gc: &Conf, sc: &ConfPred) -> Result<bool, Error> {
         }
         // Plus emerge.log after main process start time, if we didn't see specific processes
         if einfo.pkgs.is_empty() {
-            for (p, t) in started.iter() {
-                if *t > einfo.start && !r.contains(p) {
-                    r.push(p.clone())
+            for (p, (t, b)) in started.iter() {
+                if *t > einfo.start && r.iter().all(|r| r.ebuild_version() != p) {
+                    r.push(Pkg::try_new(p, *b).expect("started key should parse as Pkg"))
                 }
             }
         }
@@ -360,21 +505,22 @@ pub fn cmd_predict(gc: &Conf, sc: &ConfPred) -> Result<bool, Error> {
     for p in pkgs {
         totcount += 1;
         // Find the elapsed time, if currently running
-        let elapsed = match started.remove(&p) {
-            Some(s) if einfo.pkgs.contains(&p) => now - s,
-            Some(s) if einfo.pkgs.is_empty() && s > einfo.start => now - s,
+        let elapsed = match started.remove(p.ebuild_version()) {
+            Some((s, _)) if einfo.pkgs.contains(&p) => now - s,
+            Some((s, _)) if einfo.pkgs.is_empty() && s > einfo.start => now - s,
             _ => 0,
         };
 
         // Find the predicted time and adjust counters
-        let (fmtpred, pred) = match times.get(p.ebuild()) {
+        let (fmtpred, pred) = match times.get(&(p.ebuild().to_string(), p.bin)) {
             Some(tv) => {
                 let pred = tv.pred(sc.lim, sc.avg);
                 (pred, pred)
             },
             None => {
                 totunknown += 1;
-                (-1, sc.unknown)
+                let u = if p.bin { sc.unknownb } else { sc.unknownc };
+                (i64::MIN + u, u)
             },
         };
         totpredict += std::cmp::max(0, pred - elapsed);
@@ -383,15 +529,22 @@ pub fn cmd_predict(gc: &Conf, sc: &ConfPred) -> Result<bool, Error> {
         // Done
         if sc.show.merge && totcount <= sc.first {
             if elapsed > 0 {
-                let stage = get_buildlog(&p, &tmpdirs).unwrap_or_default();
-                tbl.row([&[&gc.pkg, &p.ebuild_version()],
+                let stage = get_buildlog(&p, &sc.tmpdirs).unwrap_or_default();
+                tbl.row([&[if p.bin { &gc.binpkg } else { &gc.pkg }, &p.ebuild_version()],
                          &[&FmtDur(fmtpred)],
                          &[&gc.clr, &"- ", &FmtDur(elapsed), &gc.clr, &stage]]);
             } else {
-                tbl.row([&[&gc.pkg, &p.ebuild_version()], &[&FmtDur(fmtpred)], &[]]);
+                tbl.row([&[if p.bin { &gc.binpkg } else { &gc.pkg }, &p.ebuild_version()],
+                         &[&FmtDur(fmtpred)],
+                         &[]]);
             }
         }
     }
+    let lastskip = totcount.saturating_sub(sc.first);
+    if sc.show.merge && gc.showskip && lastskip > 0 {
+        tbl.skiprow(&[&gc.skip, &"(skip last ", &lastskip, &")"]);
+    }
+    // Print summary line
     if totcount > 0 {
         if sc.show.tot {
             let mut s: Vec<&dyn Disp> = vec![&"Estimate for ",
@@ -402,10 +555,6 @@ pub fn cmd_predict(gc: &Conf, sc: &ConfPred) -> Result<bool, Error> {
             if totunknown > 0 {
                 s.extend::<[&dyn Disp; 5]>([&", ", &gc.cnt, &totunknown, &gc.clr, &" unknown"]);
             }
-            let tothidden = totcount.saturating_sub(sc.first.min(last - 1));
-            if tothidden > 0 {
-                s.extend::<[&dyn Disp; 5]>([&", ", &gc.cnt, &tothidden, &gc.clr, &" hidden"]);
-            }
             let e = FmtDur(totelapsed);
             if totelapsed > 0 {
                 s.extend::<[&dyn Disp; 4]>([&", ", &e, &gc.clr, &" elapsed"]);
@@ -420,14 +569,14 @@ pub fn cmd_predict(gc: &Conf, sc: &ConfPred) -> Result<bool, Error> {
     Ok(totcount > 0)
 }
 
-pub fn cmd_accuracy(gc: &Conf, sc: &ConfAccuracy) -> Result<bool, Error> {
+pub fn cmd_accuracy(gc: Conf, sc: ConfAccuracy) -> Result<bool, Error> {
     let hist = get_hist(&gc.logfile, gc.from, gc.to, Show::m(), &sc.search, sc.exact)?;
     let mut pkg_starts: HashMap<String, i64> = HashMap::new();
     let mut pkg_times: BTreeMap<String, Times> = BTreeMap::new();
     let mut pkg_errs: BTreeMap<String, Vec<f64>> = BTreeMap::new();
     let mut found = false;
-    let mut tbl = Table::new(gc).align_left(0).align_left(1).last(sc.last);
-    tbl.header(["Date", "Package", "Real", "Predicted", "Error"]);
+    let h = ["Date", "Package", "Real", "Predicted", "Error"];
+    let mut tbl = Table::new(&gc).align_left(0).align_left(1).last(sc.last).header(h);
     for p in hist {
         match p {
             Hist::MergeStart { ts, key, .. } => {
@@ -470,8 +619,7 @@ pub fn cmd_accuracy(gc: &Conf, sc: &ConfAccuracy) -> Result<bool, Error> {
     }
     drop(tbl);
     if sc.show.tot {
-        let mut tbl = Table::new(gc).align_left(0);
-        tbl.header(["Package", "Error"]);
+        let mut tbl = Table::new(&gc).align_left(0).header(["Package", "Error"]);
         for (p, e) in pkg_errs {
             let avg = e.iter().sum::<f64>() / e.len() as f64;
             tbl.row([&[&gc.pkg, &p], &[&gc.cnt, &format!("{avg:.1}%")]]);
@@ -480,7 +628,7 @@ pub fn cmd_accuracy(gc: &Conf, sc: &ConfAccuracy) -> Result<bool, Error> {
     Ok(found)
 }
 
-pub fn cmd_complete(gc: &Conf, sc: &ConfComplete) -> Result<bool, Error> {
+pub fn cmd_complete(gc: Conf, sc: ConfComplete) -> Result<bool, Error> {
     // Generate standard clap completions
     #[cfg(feature = "clap_complete")]
     if let Some(s) = &sc.shell {
@@ -490,15 +638,15 @@ pub fn cmd_complete(gc: &Conf, sc: &ConfComplete) -> Result<bool, Error> {
         return Ok(true);
     }
     // Look for (un)merged matching packages in the log and print each once
-    let term: Vec<_> = sc.pkg.iter().cloned().collect();
+    let term: Vec<_> = sc.pkg.map_or(vec![], |p| vec![p]);
     let hist = get_hist(&gc.logfile, gc.from, gc.to, Show::m(), &term, false)?;
     let mut pkgs: HashSet<String> = HashSet::new();
     for p in hist {
         if let Hist::MergeStart { .. } = p {
-            let e = p.ebuild();
-            if !pkgs.contains(e) {
+            let e = p.take_ebuild();
+            if !pkgs.contains(&e) {
                 println!("{}", e);
-                pkgs.insert(e.to_string());
+                pkgs.insert(e);
             }
         }
     }
@@ -508,9 +656,11 @@ pub fn cmd_complete(gc: &Conf, sc: &ConfComplete) -> Result<bool, Error> {
 
 #[cfg(test)]
 mod tests {
+    use super::*;
+    use crate::parse::procs;
+
     #[test]
     fn averages() {
-        use super::Times;
         use crate::Average::*;
         for (a, m, wa, wm, lim, vals) in
             [(-1, -1, -1, -1, 10, vec![]),
@@ -531,4 +681,46 @@ mod tests {
             assert_eq!(wm, t.pred(lim, WeightedMedian), "weighted median {lim} {vals:?}");
         }
     }
+
+    /// Shows the whole system's processes.
+    /// Mainly useful as an interactive test, use `cargo test -- --nocapture procs_pid1`.
+    #[test]
+    fn procs_pid1() {
+        let (gc, mut sc) = ConfPred::from_str("emlop p --pdepth 4");
+        let mut tbl = Table::new(&gc).align_left(0).align_left(2).margin(2, " ");
+        let now = epoch_now();
+        let procs = get_all_proc(&mut sc.tmpdirs);
+        proc_rows(now, &mut tbl, &procs, 1, 0, &gc, &sc);
+        println!("{}", tbl.to_string());
+    }
+
+    /// Check indentation and skipping
+    #[test]
+    fn procs_hierarchy() {
+        let (gc, sc) = ConfPred::from_str("emlop p --pdepth 3 --color=n --output=c --showskip");
+        let mut tbl = Table::new(&gc).align_left(0).align_left(2).margin(2, " ");
+        let procs = procs(&[(ProcKind::Other, "a", 1, 0),
+                            (ProcKind::Other, "a.a", 2, 1),
+                            (ProcKind::Other, "a.b", 3, 1),
+                            (ProcKind::Other, "a.a.a", 4, 2),
+                            (ProcKind::Other, "a.a.b", 5, 2),
+                            (ProcKind::Other, "a.b.a", 6, 3),
+                            // basic skip
+                            (ProcKind::Other, "a.a.a.a", 7, 4),
+                            // nested/sibling skip
+                            (ProcKind::Other, "a.a.b.a", 8, 5),
+                            (ProcKind::Other, "a.a.b.a.a", 9, 8),
+                            (ProcKind::Other, "a.a.b.b", 10, 5)]);
+        let out = r#"1 a                   9
+  2 a.a               8
+    4 a.a.a           6
+      (skip 1 below)   
+    5 a.a.b           5
+      (skip 3 below)   
+  3 a.b               7
+    6 a.b.a           4
+"#;
+        proc_rows(10, &mut tbl, &procs, 1, 0, &gc, &sc);
+        assert_eq!(tbl.to_string(), out);
+    }
 }
