@@ -6,7 +6,9 @@ mod toml;
 mod types;
 
 pub use crate::config::{cli::*, types::*};
-use crate::{config::toml::Toml, parse::AnsiStr, *};
+use crate::{config::toml::Toml,
+            parse::{AnsiStr, Theme},
+            *};
 use clap::ArgMatches;
 use std::{io::IsTerminal, path::PathBuf};
 
@@ -23,20 +25,26 @@ pub enum Configs {
 /// Using raw `set/clear` ANSI colors instead of some `paint()` method to simplify alignment.
 pub struct Conf {
     pub pkg: AnsiStr,
+    pub binpkg: AnsiStr,
     pub merge: AnsiStr,
+    pub binmerge: AnsiStr,
     pub unmerge: AnsiStr,
+    pub sync: AnsiStr,
     pub dur: AnsiStr,
     pub cnt: AnsiStr,
+    pub qmark: AnsiStr,
+    pub skip: AnsiStr,
     pub clr: AnsiStr,
     pub lineend: &'static [u8],
     pub header: bool,
+    pub showskip: bool,
     pub dur_t: DurationStyle,
     pub date_offset: time::UtcOffset,
     pub date_fmt: DateStyle,
     pub out: OutStyle,
     pub logfile: String,
-    pub from: Option<i64>,
-    pub to: Option<i64>,
+    pub from: TimeBound,
+    pub to: TimeBound,
 }
 pub struct ConfLog {
     pub show: Show,
@@ -53,8 +61,11 @@ pub struct ConfPred {
     pub last: usize,
     pub lim: u16,
     pub resume: ResumeKind,
-    pub unknown: i64,
+    pub unknownb: i64,
+    pub unknownc: i64,
     pub tmpdirs: Vec<PathBuf>,
+    pub pwidth: usize,
+    pub pdepth: usize,
 }
 pub struct ConfStats {
     pub show: Show,
@@ -114,10 +125,10 @@ fn sel<T, A, R>(cli: Option<&String>,
                 -> Result<R, ArgError>
     where R: ArgParse<String, A> + ArgParse<T, A>
 {
-    if let Some(a) = cli {
-        R::parse(a, arg, clisrc)
-    } else if let Some(a) = toml {
-        R::parse(a, arg, tomlsrc)
+    if let Some(val) = cli {
+        R::parse(val, arg, clisrc)
+    } else if let Some(val) = toml {
+        R::parse(val, arg, tomlsrc)
     } else {
         Ok(def)
     }
@@ -148,19 +159,28 @@ impl Conf {
         let color = sel!(cli, toml, color, isterm, isterm)?;
         let outdef = if isterm { OutStyle::Columns } else { OutStyle::Tab };
         let offset = get_offset(sel!(cli, toml, utc, (), false)?);
+        let theme = Theme::new().update(toml.theme.as_ref(), "theme")?
+                                .update(cli.get_one("theme"), "--theme")?;
         Ok(Self { logfile: sel!(cli, toml, logfile, (), String::from("/var/log/emerge.log"))?,
-                  from: cli.get_one("from")
-                           .map(|d| i64::parse(d, offset, "--from"))
-                           .transpose()?,
-                  to: cli.get_one("to").map(|d| i64::parse(d, offset, "--to")).transpose()?,
-                  pkg: AnsiStr::from(if color { "\x1B[1;32m" } else { "" }),
-                  merge: AnsiStr::from(if color { "\x1B[1;32m" } else { ">>> " }),
-                  unmerge: AnsiStr::from(if color { "\x1B[1;31m" } else { "<<< " }),
-                  dur: AnsiStr::from(if color { "\x1B[1;35m" } else { "" }),
-                  cnt: AnsiStr::from(if color { "\x1B[2;33m" } else { "" }),
+                  from:
+                      cli.get_one("from")
+                         .map_or(Ok(TimeBound::None), |d| TimeBound::parse(d, offset, "--from"))?,
+                  to: cli.get_one("to")
+                         .map_or(Ok(TimeBound::None), |d| TimeBound::parse(d, offset, "--to"))?,
+                  pkg: AnsiStr::from(if color { theme.merge } else { "" }),
+                  binpkg: AnsiStr::from(if color { theme.binmerge } else { "" }),
+                  merge: AnsiStr::from(if color { theme.merge } else { ">>> " }),
+                  binmerge: AnsiStr::from(if color { theme.binmerge } else { ">>> " }),
+                  unmerge: AnsiStr::from(if color { theme.unmerge } else { "<<< " }),
+                  sync: AnsiStr::from(if color { theme.sync } else { "" }),
+                  dur: AnsiStr::from(if color { theme.duration } else { "" }),
+                  cnt: AnsiStr::from(if color { theme.count } else { "" }),
+                  qmark: AnsiStr::from(if color { theme.qmark } else { "" }),
+                  skip: AnsiStr::from(if color { theme.skip } else { "" }),
                   clr: AnsiStr::from(if color { "\x1B[m" } else { "" }),
                   lineend: if color { b"\x1B[m\n" } else { b"\n" },
                   header: sel!(cli, toml, header, (), false)?,
+                  showskip: sel!(cli, toml, showskip, (), false)?,
                   dur_t: sel!(cli, toml, duration, (), DurationStyle::Hms)?,
                   date_offset: offset,
                   date_fmt: sel!(cli, toml, date, (), DateStyle::default())?,
@@ -175,7 +195,7 @@ impl Conf {
 
 impl ConfLog {
     fn try_new(cli: &ArgMatches, toml: &Toml) -> Result<Self, Error> {
-        Ok(Self { show: sel!(cli, toml, log, show, "musa", Show::m())?,
+        Ok(Self { show: sel!(cli, toml, log, show, "rmusa", Show::m())?,
                   search: cli.get_many("search").unwrap_or_default().cloned().collect(),
                   exact: cli.get_flag("exact"),
                   starttime: sel!(cli, toml, log, starttime, (), false)?,
@@ -193,23 +213,32 @@ impl ConfPred {
         } else {
             vec![PathBuf::from("/var/tmp")]
         };
-        Ok(Self { show: sel!(cli, toml, predict, show, "emta", Show::emt())?,
+        Ok(Self { show: sel!(cli, toml, predict, show, "rmta", Show::rmt())?,
                   avg: sel!(cli, toml, predict, avg, (), Average::Median)?,
-                  lim: sel!(cli, toml, predict, limit, 1..65000, 10)? as u16,
-                  unknown: sel!(cli, toml, predict, unknown, 0..3600, 10)?,
+                  lim: sel!(cli, toml, predict, limit, 1..=65000, 10)? as u16,
+                  unknownb: sel!(cli, toml, predict, unknownb, 0..=3600, 10)?,
+                  unknownc: sel!(cli, toml, predict, unknownc, 0..=3600, 30)?,
                   resume: *cli.get_one("resume").unwrap_or(&ResumeKind::Auto),
                   tmpdirs,
                   first: *cli.get_one("first").unwrap_or(&usize::MAX),
-                  last: *cli.get_one("last").unwrap_or(&usize::MAX) })
+                  last: *cli.get_one("last").unwrap_or(&usize::MAX),
+                  pwidth: sel!(cli, toml, predict, pwidth, 10..=1000, 60)? as usize,
+                  pdepth: sel!(cli, toml, predict, pdepth, 0..=100, 3)? as usize })
+    }
+    #[cfg(test)]
+    pub fn from_str(s: impl AsRef<str>) -> (Conf, Self) {
+        let cli = cli::build_cli().get_matches_from(s.as_ref().split_whitespace());
+        (Conf::try_new(&cli, &Toml::default()).unwrap(),
+         ConfPred::try_new(cli.subcommand().unwrap().1, &Toml::default()).unwrap())
     }
 }
 
 impl ConfStats {
     fn try_new(cli: &ArgMatches, toml: &Toml) -> Result<Self, Error> {
-        Ok(Self { show: sel!(cli, toml, stats, show, "ptsa", Show::p())?,
+        Ok(Self { show: sel!(cli, toml, stats, show, "rptsa", Show::p())?,
                   search: cli.get_many("search").unwrap_or_default().cloned().collect(),
                   exact: cli.get_flag("exact"),
-                  lim: sel!(cli, toml, stats, limit, 1..65000, 10)? as u16,
+                  lim: sel!(cli, toml, stats, limit, 1..=65000, 10)? as u16,
                   avg: sel!(cli, toml, stats, avg, (), Average::Median)?,
                   group: sel!(cli, toml, stats, group, (), Timespan::None)? })
     }
@@ -221,7 +250,7 @@ impl ConfAccuracy {
                   search: cli.get_many("search").unwrap_or_default().cloned().collect(),
                   exact: cli.get_flag("exact"),
                   avg: sel!(cli, toml, accuracy, avg, (), Average::Median)?,
-                  lim: sel!(cli, toml, accuracy, limit, 1..65000, 10)? as u16,
+                  lim: sel!(cli, toml, accuracy, limit, 1..=65000, 10)? as u16,
                   last: *cli.get_one("last").unwrap_or(&usize::MAX) })
     }
 }
