@@ -30,36 +30,40 @@ pub fn build_cli() -> Command {
                                              virtual/rust: Matches only `virtual/rust`\n  \
                                              RuSt:         Matches nothing (case-sensitive)\n  \
                                              ru:           Matches nothing (whole name only)");
-    let show_l = Arg::new("show").short('s')
-                                 .long("show")
-                                 .value_name("m,u,s,a")
-                                 .display_order(3)
-                                 .help_heading("Filter")
-                                 .help("Show (m)erges, (u)nmerges, (s)yncs, and/or (a)ll")
-                                 .long_help("Show (any combination of)\n  \
-                                             m: Package merges\n  \
-                                             u: Package unmerges\n  \
-                                             s: Repository syncs\n  \
-                                             a: All of the above");
-    let show_s = Arg::new("show").short('s')
-                                 .long("show")
-                                 .value_name("p,t,s,a")
-                                 .display_order(3)
-                                 .help_heading("Filter")
-                                 .help("Show (p)ackages, (t)otals, (s)yncs, and/or (a)ll")
-                                 .long_help("Show (any combination of)\n  \
-                                             p: Individual package merges/unmerges\n  \
-                                             t: Total package merges/unmerges\n  \
-                                             s: Repository syncs\n  \
-                                             a: All of the above");
+    let show_l =
+        Arg::new("show").short('s')
+                        .long("show")
+                        .value_name("r,m,u,s,a")
+                        .display_order(3)
+                        .help_heading("Filter")
+                        .help("Show emerge (r)uns, (m)erges, (u)nmerges, (s)yncs, and/or (a)ll")
+                        .long_help("Show (any combination of)\n  \
+                                    r: Emerge runs\n  \
+                                    m: Package merges\n  \
+                                    u: Package unmerges\n  \
+                                    s: Repository syncs\n  \
+                                    a: All of the above");
+    let show_s =
+        Arg::new("show").short('s')
+                        .long("show")
+                        .value_name("r,p,t,s,a")
+                        .display_order(3)
+                        .help_heading("Filter")
+                        .help("Show emerge (r)uns, (p)ackages, (t)otals, (s)yncs, and/or (a)ll")
+                        .long_help("Show (any combination of)\n  \
+                                    r: Emerge runs\n  \
+                                    p: Individual package merges/unmerges\n  \
+                                    t: Total package merges/unmerges\n  \
+                                    s: Repository syncs\n  \
+                                    a: All of the above");
     let show_p = Arg::new("show").short('s')
                                  .long("show")
-                                 .value_name("e,m,t,a")
+                                 .value_name("r,m,t,a")
                                  .display_order(3)
                                  .help_heading("Filter")
-                                 .help("Show (e)emerge processes, (m)erges, (t)otal, and/or (a)ll")
+                                 .help("Show (r)unning processes, (m)erges, (t)otal, and/or (a)ll")
                                  .long_help("Show (any combination of)\n  \
-                                             e: Current emerge processes\n  \
+                                             r: Running emerge processes\n  \
                                              m: Package merges\n  \
                                              t: Total estimate\n  \
                                              a: All of the above");
@@ -73,10 +77,11 @@ pub fn build_cli() -> Command {
                                              m: Package merges\n  \
                                              t: Totals\n  \
                                              a: All of the above");
-    let h = "Only parse log entries after <date>\n  \
+    let h = "Only parse log entries after <date/command>\n  \
              2018-03-04|2018-03-04 12:34:56|2018-03-04T12:34: Absolute ISO date\n  \
              123456789:                                       Absolute unix timestamp\n  \
-             1 year, 2 months|10d:                            Relative date";
+             1 year, 2 months|10d:                            Relative date\n  \
+             1c|2 commands|c                                  Nth emerge command";
     let from = Arg::new("from").short('f')
                                .long("from")
                                .value_name("date")
@@ -86,10 +91,11 @@ pub fn build_cli() -> Command {
                                .help_heading("Filter")
                                .help(h.split_once('\n').unwrap().0)
                                .long_help(h);
-    let h = "Only parse log entries before <date>\n  \
+    let h = "Only parse log entries before <date/command>\n  \
              2018-03-04|2018-03-04 12:34:56|2018-03-04T12:34: Absolute ISO date\n  \
              123456789:                                       Absolute unix timestamp\n  \
-             1 year, 2 months|10d:                            Relative date";
+             1 year, 2 months|10d:                            Relative date\n  \
+             1c|2 commands|c                                  Nth-last emerge command";
     let to = Arg::new("to").short('t')
                            .long("to")
                            .value_name("date")
@@ -121,8 +127,8 @@ pub fn build_cli() -> Command {
                                .help_heading("Filter")
                                .help("Show only the last <num> entries")
                                .long_help("Show only the last <num> entries\n  \
-                                             (empty)|1: last entry\n  \
-                                             5:         last 5 entries\n");
+                                           (empty)|1: last entry\n  \
+                                           5:         last 5 entries\n");
     let h = "Use main, backup, either, or no portage resume list\n\
              This is ignored if STDIN is a piped `emerge -p` output\n  \
              (default)|auto|a: Use main or backup resume list, if currently emerging\n  \
@@ -160,23 +166,32 @@ pub fn build_cli() -> Command {
                                  .display_order(11)
                                  .help_heading("Stats")
                                  .help("Use the last <num> merge times to predict durations");
-    let avg =
-        Arg::new("avg").long("avg")
-                       .value_name("fn")
-                       .display_order(12)
-                       .help_heading("Stats")
-                       .help("Select function used to predict durations")
-                       .long_help("Select function used to predict durations\n  \
-                                   arith|a:            simple 'sum/count' average\n  \
-                                   (defaut)|median|m:  middle value, mitigates outliers\n  \
-                                   weighted-arith|wa:  'sum/count' with more weight for recent values\n  \
-                                   weighted-median|wm: \"middle\" value shifted toward recent values");
-    let unknown = Arg::new("unknown").long("unknown")
-                                     .num_args(1)
-                                     .value_name("secs")
-                                     .display_order(13)
-                                     .help_heading("Stats")
-                                     .help("Assume unkown packages take <secs> seconds to merge");
+    let h = "Select function used to predict durations\n  \
+             arith|a:            simple 'sum/count' average\n  \
+             (defaut)|median|m:  middle value, mitigates outliers\n  \
+             weighted-arith|wa:  'sum/count' with more weight for recent values\n  \
+             weighted-median|wm: \"middle\" value shifted toward recent values";
+    let avg = Arg::new("avg").long("avg")
+                             .value_name("fn")
+                             .display_order(12)
+                             .help_heading("Stats")
+                             .help(h.split_once('\n').unwrap().0)
+                             .long_help(h);
+    let unknownc =
+        Arg::new("unknownc").long("unknownc")
+                            .alias("unknown")
+                            .num_args(1)
+                            .value_name("secs")
+                            .display_order(13)
+                            .help_heading("Stats")
+                            .help("Assume unkown compiled packages take <secs> seconds to merge");
+    let unknownb =
+        Arg::new("unknownb").long("unknownb")
+                            .num_args(1)
+                            .value_name("secs")
+                            .display_order(14)
+                            .help_heading("Stats")
+                            .help("Assume unkown binary packages take <secs> seconds to merge");
 
     ////////////////////////////////////////////////////////////
     // Format arguments
@@ -201,21 +216,21 @@ pub fn build_cli() -> Command {
                                                    hmsfixed:      0:10:30\n  \
                                                    secs|s:        630\n  \
                                                    human|h:       10 minutes, 30 seconds");
-    let date =
-        Arg::new("date").long("date")
-                        .value_name("format")
-                        .global(true)
-                        .display_order(22)
-                        .help_heading("Format")
-                        .help("Output dates in different formats")
-                        .long_help("Output dates in different formats\n  \
-                                    ymd|d:               2022-01-31\n  \
-                                    (default)|ymdhms|dt: 2022-01-31 08:59:46\n  \
-                                    ymdhmso|dto:         2022-01-31 08:59:46 +00:00\n  \
-                                    rfc3339|3339:        2022-01-31T08:59:46+00:00\n  \
-                                    rfc2822|2822:        Mon, 31 Jan 2022 08:59:46 +00:00\n  \
-                                    compact:             20220131085946\n  \
-                                    unix:                1643619586");
+    let h = "Output dates in different formats\n  \
+             ymd|d:               2022-01-31\n  \
+             (default)|ymdhms|dt: 2022-01-31 08:59:46\n  \
+             ymdhmso|dto:         2022-01-31 08:59:46 +00:00\n  \
+             rfc3339|3339:        2022-01-31T08:59:46+00:00\n  \
+             rfc2822|2822:        Mon, 31 Jan 2022 08:59:46 +00:00\n  \
+             compact:             20220131085946\n  \
+             unix:                1643619586";
+    let date = Arg::new("date").long("date")
+                               .value_name("format")
+                               .global(true)
+                               .display_order(22)
+                               .help_heading("Format")
+                               .help(h.split_once('\n').unwrap().0)
+                               .long_help(h);
     let utc = Arg::new("utc").long("utc")
                              .value_name("bool")
                              .global(true)
@@ -231,29 +246,70 @@ pub fn build_cli() -> Command {
                                          .display_order(24)
                                          .help_heading("Format")
                                          .help("Display start time instead of end time");
+    let pwidth = Arg::new("pwidth").long("pwidth")
+                                   .short('W')
+                                   .value_name("num")
+                                   .num_args(1)
+                                   .display_order(25)
+                                   .help_heading("Format")
+                                   .help("Maximum width of emerge proces commandline (default 60)");
+    let pdepth = Arg::new("pdepth").long("pdepth")
+                                   .short('D')
+                                   .value_name("num")
+                                   .num_args(1)
+                                   .display_order(26)
+                                   .help_heading("Format")
+                                   .help("Maximum depth of emerge proces tree (default 3)");
     let color = Arg::new("color").long("color")
                                  .value_name("bool")
                                  .global(true)
                                  .num_args(..=1)
                                  .default_missing_value("y")
-                                 .display_order(25)
+                                 .display_order(27)
                                  .help_heading("Format")
                                  .help("Enable color (yes/no/auto)")
                                  .long_help("Enable color (yes/no/auto)\n  \
                                              (default)|auto|a: colored if on tty\n  \
                                              (empty)|yes|y:    colored\n  \
                                              no|n:             not colored");
-    let output = Arg::new("output").short('o')
-                                   .long("output")
+    let h = "Set terminal colors\n\
+             Argument should be a space-separated list of <key>:<SGR> strings, where\n  \
+             <key> is one of merge, binmerge, unmerge, sync, duration, qmark, or skip\n  \
+             <SGR> is an Ansi SGR code (\
+             https://en.wikipedia.org/wiki/ANSI_escape_code#Select_Graphic_Rendition_parameters)\n\
+             Eg: \"count:0 duration:1;3;37\" sets counts to unstyled and durations to bright italic white";
+    let theme = Arg::new("theme").long("theme")
+                                 .value_name("key:SGR")
+                                 .global(true)
+                                 .num_args(1)
+                                 .display_order(28)
+                                 .help_heading("Format")
+                                 .help("Set terminal colors")
+                                 .long_help(h);
+    let output = Arg::new("output").long("output")
+                                   .short('o')
                                    .value_name("format")
                                    .global(true)
-                                   .display_order(26)
+                                   .display_order(29)
                                    .help_heading("Format")
                                    .help("Ouput format (columns/tab/auto)")
                                    .long_help("Ouput format (columns/tab/auto)\n  \
                                                (default)|auto|a: columns on tty, tab otherwise\n  \
                                                columns|c:        space-aligned columns\n  \
                                                tab|t:            tab-separated values");
+    let h = "Show number of skipped rows (yes/no)\n  \
+             (empty)|yes|y: Show 'skip <num>' placeholder\n  \
+             no|n:          Skip rows silently";
+    let showskip = Arg::new("showskip").long("showskip")
+                                       .short('S')
+                                       .value_name("bool")
+                                       .global(true)
+                                       .num_args(..=1)
+                                       .default_missing_value("y")
+                                       .display_order(30)
+                                       .help_heading("Format")
+                                       .help(h.split_once('\n').unwrap().0)
+                                       .long_help(h);
 
     ////////////////////////////////////////////////////////////
     // Misc arguments
@@ -263,14 +319,14 @@ pub fn build_cli() -> Command {
                                      .value_name("file")
                                      .global(true)
                                      .num_args(1)
-                                     .display_order(30)
+                                     .display_order(40)
                                      .help("Location of emerge log file");
     let tmpdir = Arg::new("tmpdir").long("tmpdir")
                                    .value_name("dir")
                                    .num_args(1)
                                    .action(Append)
                                    .value_parser(value_parser!(PathBuf))
-                                   .display_order(31)
+                                   .display_order(41)
                                    .help("Location of portage tmpdir")
                                    .long_help("Location of portage tmpdir\n\
                                                Multiple folders can be provided\n\
@@ -278,7 +334,7 @@ pub fn build_cli() -> Command {
     let verbose = Arg::new("verbose").short('v')
                                      .global(true)
                                      .action(Count)
-                                     .display_order(33)
+                                     .display_order(43)
                                      .help("Increase verbosity (can be given multiple times)")
                                      .long_help("Increase verbosity (defaults to errors only)\n  \
                                                  -v:   show warnings\n  \
@@ -289,14 +345,14 @@ pub fn build_cli() -> Command {
         Arg::new("shell").long("shell")
                          .help("Write generated (development) <shell> completion script to stdout")
                          .num_args(1)
-                         .display_order(34);
+                         .display_order(44);
     let h = "List matching packages from emerge.log\n\
              Uses the same semantics as `log <search>` filtering. \
              An empty search lists everything.";
     let onepkg = Arg::new("pkg").help(h.split_once('\n').unwrap().0)
                                 .long_help(h)
                                 .num_args(1)
-                                .display_order(35);
+                                .display_order(45);
 
     ////////////////////////////////////////////////////////////
     // Subcommands
@@ -323,7 +379,10 @@ pub fn build_cli() -> Command {
                                           .arg(&last)
                                           .arg(tmpdir)
                                           .arg(resume)
-                                          .arg(unknown)
+                                          .arg(unknownc)
+                                          .arg(unknownb)
+                                          .arg(pwidth)
+                                          .arg(pdepth)
                                           .arg(&avg)
                                           .arg(&limit);
     let h = "Show statistics about syncs, per-package (un)merges, and total (un)merges\n\
@@ -392,6 +451,8 @@ pub fn build_cli() -> Command {
                          .arg(output)
                          .arg(logfile)
                          .arg(verbose)
+                         .arg(showskip)
+                         .arg(theme)
                          .subcommand(cmd_log)
                          .subcommand(cmd_pred)
                          .subcommand(cmd_stats)
