@@ -1,7 +1,7 @@
 use crate::{config::{ArgError, ArgParse},
             table::Disp,
             wtb, Conf, DurationStyle};
-use anyhow::{bail, Error};
+use anyhow::{bail, ensure, Error};
 use log::{debug, warn};
 use regex::Regex;
 use std::{io::Write as _,
@@ -74,75 +74,103 @@ pub fn epoch_now() -> i64 {
     SystemTime::now().duration_since(UNIX_EPOCH).unwrap().as_secs() as i64
 }
 
+#[cfg_attr(test, derive(PartialEq, Debug))]
+#[derive(Clone, Copy)]
+pub enum TimeBound {
+    /// Unbounded
+    None,
+    /// Bound by unix timestamp
+    Unix(i64),
+    /// Bound by time of nth fist/last emerge run
+    Run(usize),
+}
+
 /// Parse datetime in various formats, returning unix timestamp
-impl ArgParse<String, UtcOffset> for i64 {
+impl ArgParse<String, UtcOffset> for TimeBound {
     fn parse(val: &String, offset: UtcOffset, src: &'static str) -> Result<Self, ArgError> {
         let s = val.trim();
         let et = match i64::from_str(s) {
-            Ok(i) => return Ok(i),
+            Ok(i) => return Ok(Self::Unix(i)),
             Err(et) => et,
         };
         let ea = match parse_date_yyyymmdd(s, offset) {
-            Ok(i) => return Ok(i),
+            Ok(i) => return Ok(Self::Unix(i)),
+            Err(ea) => ea,
+        };
+        let ec = match parse_command_num(s) {
+            Ok(i) => return Ok(Self::Run(i)),
             Err(ea) => ea,
         };
         match parse_date_ago(s) {
-            Ok(i) => Ok(i),
+            Ok(i) => Ok(Self::Unix(i)),
             Err(er) => {
-                let m = format!("Not a unix timestamp ({et}), absolute date ({ea}), or relative date ({er})");
+                let m = format!("Not a unix timestamp ({et}), absolute date ({ea}), relative date ({er}), or command ({ec})");
                 Err(ArgError::new(val, src).msg(m))
             },
         }
     }
 }
 
+/// Parse a command index (parse as 1-based, return as 0-based)
+fn parse_command_num(s: &str) -> Result<usize, Error> {
+    use atoi::FromRadix10;
+    let (num, pos) = usize::from_radix_10(s.as_bytes());
+    match s[pos..].trim() {
+        "c" | "command" | "commands" if num > 0 => Ok(num - 1),
+        "c" | "command" if pos == 0 => Ok(0),
+        _ => bail!("bad span {:?}", &s[pos..]),
+    }
+}
+
 /// Parse a number of day/years/hours/etc in the past, relative to current time
 fn parse_date_ago(s: &str) -> Result<i64, Error> {
-    if !s.chars().all(|c| c.is_alphanumeric() || c == ' ' || c == ',') {
-        bail!("Illegal char");
-    }
+    ensure!(s.chars().all(|c| c.is_ascii_alphanumeric() || c == ' ' || c == ','), "bad char");
+    ensure!(s.chars().any(|c| c.is_ascii_alphabetic()), "empty");
     let mut now = OffsetDateTime::now_utc();
-    let re = Regex::new("([0-9]+|[a-z]+)").expect("Bad date span regex");
-    let mut tokens = re.find_iter(s);
-    let mut at_least_one = false;
+
+    // Handle case where only a span is given
+    if let Ok(now) = parse_date_span(1, s.trim(), now) {
+        return Ok(now.unix_timestamp());
+    }
 
     // The regex gives us a list of positive integers and strings. We expect to always have a
     // number, followed by a known string.
+    let re = Regex::new("([0-9]+|[a-z]+)").expect("Bad date span regex");
+    let mut tokens = re.find_iter(s);
     while let Some(t) = tokens.next() {
-        at_least_one = true;
-        let num: i32 = t.as_str().parse()?;
-        match tokens.next().map(|m| m.as_str()).unwrap_or("") {
-            "y" | "year" | "years" => {
-                let d = Date::from_calendar_date(now.year() - num, now.month(), now.day())?;
-                now = now.replace_date(d);
-            },
-            "m" | "month" | "months" => {
-                let mut month = now.month();
-                let mut year = now.year();
-                for _ in 0..num {
-                    month = month.previous();
-                    if month == time::Month::December {
-                        year -= 1;
-                    }
-                }
-                let d = Date::from_calendar_date(year, month, now.day())?;
-                now = now.replace_date(d);
-            },
-            "w" | "week" | "weeks" => now -= num * Duration::WEEK,
-            "d" | "day" | "days" => now -= num * Duration::DAY,
-            "h" | "hour" | "hours" => now -= num * Duration::HOUR,
-            "min" | "mins" | "minute" | "minutes" => now -= num * Duration::MINUTE,
-            "s" | "sec" | "secs" | "second" | "seconds" => now -= num * Duration::SECOND,
-            o => bail!("bad span {:?}", o),
-        };
-    }
-
-    if !at_least_one {
-        bail!("No token found");
+        let num: i32 = t.as_str().parse().map_err(|_| Error::msg("not a number"))?;
+        now = parse_date_span(num, tokens.next().map(|m| m.as_str()).unwrap_or(""), now)?;
     }
     Ok(now.unix_timestamp())
 }
 
+fn parse_date_span(num: i32, span: &str, now: OffsetDateTime) -> Result<OffsetDateTime, Error> {
+    Ok(match span {
+        "y" | "year" | "years" => {
+            let d = Date::from_calendar_date(now.year() - num, now.month(), now.day())?;
+            now.replace_date(d)
+        },
+        "m" | "month" | "months" => {
+            let mut month = now.month();
+            let mut year = now.year();
+            for _ in 0..num {
+                month = month.previous();
+                if month == time::Month::December {
+                    year -= 1;
+                }
+            }
+            let d = Date::from_calendar_date(year, month, now.day())?;
+            now.replace_date(d)
+        },
+        "w" | "week" | "weeks" => now - num * Duration::WEEK,
+        "d" | "day" | "days" => now - num * Duration::DAY,
+        "h" | "hour" | "hours" => now - num * Duration::HOUR,
+        "min" | "mins" | "minute" | "minutes" => now - num * Duration::MINUTE,
+        "s" | "sec" | "secs" | "second" | "seconds" => now - num * Duration::SECOND,
+        o => bail!("bad span {:?}", o),
+    })
+}
+
 /// Parse rfc3339-like format with added flexibility
 fn parse_date_yyyymmdd(s: &str, offset: UtcOffset) -> Result<i64, Error> {
     use time::format_description::{modifier::*, Component, FormatItem::*};
@@ -180,7 +208,7 @@ fn parse_date_yyyymmdd(s: &str, offset: UtcOffset) -> Result<i64, Error> {
         ]))
     ])?;
     if !rest.is_empty() {
-        bail!("Junk at end")
+        bail!("junk at end")
     }
     Ok(OffsetDateTime::try_from(p)?.unix_timestamp())
 }
@@ -258,6 +286,9 @@ impl Timespan {
 }
 
 /// Wrapper around a duration (seconds) to implement `table::Disp`
+///
+/// Normal negatives (> -2^62) are rendered as `?`
+/// Far negatives (< -2^62) are rendered as `{value + 1 + 2^63}?`
 pub struct FmtDur(pub i64);
 impl crate::table::Disp for FmtDur {
     fn out(&self, buf: &mut Vec<u8>, conf: &Conf) -> usize {
@@ -265,9 +296,14 @@ impl crate::table::Disp for FmtDur {
         use DurationStyle::*;
         let sec = self.0;
         let dur = conf.dur.val;
+        let qmark = conf.qmark.val;
         let start = buf.len();
         match conf.dur_t {
-            _ if sec < 0 => wtb!(buf, "{dur}?"),
+            _ if sec < i64::MIN / 2 => {
+                FmtDur(i64::MAX + sec + 1).out(buf, conf);
+                wtb!(buf, "{qmark}?")
+            },
+            _ if sec < 0 => wtb!(buf, "{qmark}?"),
             Hms if sec >= 3600 => {
                 wtb!(buf, "{dur}{}:{:02}:{:02}", sec / 3600, sec % 3600 / 60, sec % 60)
             },
@@ -287,7 +323,7 @@ impl crate::table::Disp for FmtDur {
                 }
             },
         }
-        buf.len() - start - conf.dur.val.len()
+        crate::parse::Ansi::len(&buf[start..])
     }
 }
 
@@ -299,8 +335,8 @@ mod test {
     fn parse_3339(s: &str) -> OffsetDateTime {
         OffsetDateTime::parse(s, &Rfc3339).expect(s)
     }
-    fn parse_date(s: &str, o: UtcOffset) -> Result<i64, ArgError> {
-        i64::parse(&String::from(s), o, "")
+    fn parse_date(s: &str, o: UtcOffset) -> Result<TimeBound, ArgError> {
+        TimeBound::parse(&String::from(s), o, "")
     }
     fn ts(t: OffsetDateTime) -> i64 {
         t.unix_timestamp()
@@ -315,25 +351,32 @@ mod test {
         let tz_utc = UtcOffset::UTC;
 
         // Absolute dates
-        assert_eq!(Ok(then), parse_date(" 1522713600 ", tz_utc));
-        assert_eq!(Ok(then), parse_date(" 2018-04-03 ", tz_utc));
-        assert_eq!(Ok(then + hour + min), parse_date("2018-04-03 01:01", tz_utc));
-        assert_eq!(Ok(then + hour + min + 1), parse_date("2018-04-03 01:01:01", tz_utc));
-        assert_eq!(Ok(then + hour + min + 1), parse_date("2018-04-03T01:01:01", tz_utc));
+        assert_eq!(Ok(TimeBound::Unix(then)), parse_date(" 1522713600 ", tz_utc));
+        assert_eq!(Ok(TimeBound::Unix(then)), parse_date(" 2018-04-03 ", tz_utc));
+        assert_eq!(Ok(TimeBound::Unix(then + hour + min)), parse_date("2018-04-03 01:01", tz_utc));
+        assert_eq!(Ok(TimeBound::Unix(then + hour + min + 1)),
+                   parse_date("2018-04-03 01:01:01", tz_utc));
+        assert_eq!(Ok(TimeBound::Unix(then + hour + min + 1)),
+                   parse_date("2018-04-03T01:01:01", tz_utc));
 
         // Different timezone (not calling `get_utcoffset()` because tests are threaded, which makes
         // `UtcOffset::current_local_offset()` error out)
         for secs in [hour, -1 * hour, 90 * min, -90 * min] {
             let offset = dbg!(UtcOffset::from_whole_seconds(secs.try_into().unwrap()).unwrap());
-            assert_eq!(Ok(then - secs), parse_date("2018-04-03T00:00", offset));
+            assert_eq!(Ok(TimeBound::Unix(then - secs)), parse_date("2018-04-03T00:00", offset));
         }
 
         // Relative dates
-        assert_eq!(Ok(now - hour - 3 * day - 45), parse_date("1 hour, 3 days  45sec", tz_utc));
-        assert_eq!(Ok(now - 5 * 7 * day), parse_date("5 weeks", tz_utc));
+        assert_eq!(Ok(TimeBound::Unix(now - hour - 3 * day - 45)),
+                   parse_date("1 hour, 3 days  45sec", tz_utc));
+        assert_eq!(Ok(TimeBound::Unix(now - 5 * 7 * day)), parse_date("5 weeks", tz_utc));
+        assert_eq!(Ok(TimeBound::Unix(now - hour - 2 * day)), parse_date("2d1h", tz_utc));
+        assert_eq!(Ok(TimeBound::Unix(now - 7 * day)), parse_date("w", tz_utc));
 
         // Failure cases
         assert!(parse_date("", tz_utc).is_err());
+        assert!(parse_date(" ", tz_utc).is_err());
+        assert!(parse_date(",", tz_utc).is_err());
         assert!(parse_date("junk2018-04-03T01:01:01", tz_utc).is_err());
         assert!(parse_date("2018-04-03T01:01:01junk", tz_utc).is_err());
         assert!(parse_date("152271000o", tz_utc).is_err());
@@ -341,6 +384,19 @@ mod test {
         assert!(parse_date("a while ago", tz_utc).is_err());
     }
 
+    #[test]
+    fn command_num() {
+        assert_eq!(parse_command_num("1c").unwrap(), 0);
+        assert_eq!(parse_command_num("5c").unwrap(), 4);
+        assert_eq!(parse_command_num("c").unwrap(), 0);
+        assert_eq!(parse_command_num("1 commands ").unwrap(), 0);
+        assert!(parse_command_num("").is_err());
+        assert!(parse_command_num("0c").is_err());
+        assert!(parse_command_num("0").is_err());
+        assert!(parse_command_num("1cool").is_err());
+        assert!(parse_command_num("-1c").is_err());
+    }
+
     #[test]
     fn timespan_next_() {
         for t in [// input             year       month      week       day
@@ -395,7 +451,8 @@ mod test {
              ("99:59:59", "99:59:59", "359999", "4 days, 3 hours, 59 minutes, 59 seconds", 359999),
              ("100:00:00", "100:00:00", "360000", "4 days, 4 hours", 360000),
              ("?", "?", "?", "?", -1),
-             ("?", "?", "?", "?", -123456)]
+             ("?", "?", "?", "?", -123456),
+             ("42?", "0:00:42?", "42?", "42 seconds?", i64::MIN + 42)]
         {
             for (st, exp) in [("hms", hms), ("hmsfixed", fixed), ("secs", secs), ("human", human)] {
                 let mut buf = vec![];
